#define STRICT
#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <windowsx.h>
#include <commdlg.h>
#include <eplay.h>
#include "egiplay.h"

#if defined __BORLANDC__
  #pragma resource "egiplay.res"
#endif

static char const szAppName[] = "EGIplay";
static FlicAnim Anim;


BOOL  InitApplication(HINSTANCE hInst);
HWND  InitInstance(HINSTANCE hInst, int nCmdShow);

LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
void Cls_OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify);
BOOL Cls_OnCreate(HWND hwnd, CREATESTRUCT FAR* lpCreateStruct);
void Cls_OnDestroy(HWND hwnd);
void Cls_OnPaint(HWND hwnd);
void Cls_OnPaletteChanged(HWND hwnd, HWND hwndPaletteChange);
BOOL Cls_OnQueryNewPalette(HWND hwnd);


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPSTR /*lpszCmdParam*/, int nCmdShow)
{
  if (hPrevInstance == NULL && !InitApplication(hInstance))
    return FALSE;

  if (!InitInstance(hInstance, nCmdShow))
    return FALSE;

  MSG Msg;
  while (GetMessage(&Msg, NULL, 0, 0)) {
    TranslateMessage(&Msg);
    DispatchMessage( &Msg );
  } /* while */
  return Msg.wParam;
}

BOOL InitApplication(HINSTANCE hInstance)
{
  WNDCLASS wc = { 0 };
  wc.lpfnWndProc = WndProc;
  wc.hInstance = hInstance;
  wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
  wc.hCursor = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
  wc.lpszClassName = szAppName;
  wc.lpszMenuName = szAppName;
  return (RegisterClass(&wc) != 0);
}

HWND InitInstance(HINSTANCE hInstance, int nCmdShow)
{
  HWND hwnd = CreateWindow(szAppName, szAppName,
                           WS_OVERLAPPEDWINDOW,
                           CW_USEDEFAULT, CW_USEDEFAULT,
                           CW_USEDEFAULT, CW_USEDEFAULT,
                           NULL, NULL, hInstance, NULL );
  if (hwnd != NULL) {
    ShowWindow(hwnd, nCmdShow);
    UpdateWindow(hwnd);
  } /* if */
  return hwnd;
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT Message,
                         WPARAM wParam, LPARAM lParam )
{
  switch (Message) {
    HANDLE_MSG(hwnd, WM_COMMAND,         Cls_OnCommand);
//  HANDLE_MSG(hwnd, WM_CREATE,          Cls_OnCreate);
    HANDLE_MSG(hwnd, WM_DESTROY,         Cls_OnDestroy);
    HANDLE_MSG(hwnd, WM_PAINT,           Cls_OnPaint);
    HANDLE_MSG(hwnd, WM_PALETTECHANGED,  Cls_OnPaletteChanged);
    HANDLE_MSG(hwnd, WM_QUERYNEWPALETTE, Cls_OnQueryNewPalette);
  default:
    return DefWindowProc(hwnd, Message, wParam, lParam);
  } /* switch */
}

void Cls_OnCommand(HWND hwnd, int id, HWND /*hwndCtl*/, UINT /*codeNotify*/)
{
  switch (id) {
  case IDM_ABOUT:
    MessageBox(hwnd,
               "EGIplay is a simple animation player that demonstrates the "
               "EGI player API.\nIt is provided as a programming example for "
               "the EGI product.",
               szAppName, MB_OK | MB_ICONINFORMATION);
    break;

  case IDM_EXIT:
    DestroyWindow(hwnd);
    break;

  case IDM_OPEN:
    { /* local */
      OPENFILENAME ofn = { 0 };
      char Filename[MAX_PATH] = "";
      ofn.lStructSize  = sizeof ofn;
      ofn.hwndOwner    = hwnd;
      ofn.lpstrFilter  = "FLIC files\0*.flc;*.fli;*.flx\0";
      ofn.nFilterIndex = 1;
      ofn.lpstrFile    = Filename;
      ofn.nMaxFile     = sizeof Filename;
      ofn.Flags        = OFN_FILEMUSTEXIST;
      if (GetOpenFileName(&ofn)) {
        Anim.Open(Filename);
        if (Anim) {
          Anim.NextFrame();
          InvalidateRect(hwnd, NULL, FALSE);
        } else
          MessageBox(hwnd,
                     "Unable to open the requested file (probably due to an "
                     "invalid or unsupported file format).",
                     szAppName, MB_OK | MB_ICONERROR);
      } /* if */
    } /* local */
    break;

  case IDM_CLOSE:
    if (Anim)
      Anim.Close();
    break;

  case IDM_START:
    if (Anim && !Anim.IsPlaying())
      Anim.Play(hwnd);
    break;

  case IDM_STOP:
    if (Anim) {
      Anim.Stop();
      InvalidateRect(hwnd, NULL, FALSE);
    } /* if */
    break;
  } /* switch */
}

//BOOL Cls_OnCreate(HWND /*hwnd*/, CREATESTRUCT FAR * /*lpCreateStruct*/)
//{
//  return TRUE;
//}

void Cls_OnDestroy(HWND /*hwnd*/)
{
  PostQuitMessage(0);
}

void Cls_OnPaint(HWND hwnd)
{
  PAINTSTRUCT PaintStruct;
  BeginPaint(hwnd, &PaintStruct);
  if (Anim)
    Anim.Paint(PaintStruct.hdc);
  EndPaint(hwnd, &PaintStruct);
}

void Cls_OnPaletteChanged(HWND hwnd, HWND hwndPaletteChange)
{
  if (hwnd != hwndPaletteChange)
    InvalidateRect(hwnd, NULL, TRUE);
}

BOOL Cls_OnQueryNewPalette(HWND hwnd)
{
  UINT result = 0;

  if (Anim) {
    /* get the palette from the animation */
    HPALETTE hpal = Anim.GetHPalette();

    /* select and realize this palette */
    HDC hdc = GetDC(hwnd);
    hpal = SelectPalette(hdc, hpal, FALSE);
    result = RealizePalette(hdc);

    /* restore the palette (before releasing the DC) */
    SelectPalette(hdc, hpal, TRUE);
    RealizePalette(hdc);
    ReleaseDC(hwnd, hdc);
  } /* if */

  return (result > 0);
}

